@@ -25,10 +25,15 @@ struct isDouble<double>{
 #define CSKIP(...) __VA_ARGS__
 #define TMB_EXTERN
 #endif
-#ifdef TMB_PRECOMPILE
-#define IF_TMB_PRECOMPILE(...) __VA_ARGS__
+#ifdef TMB_PRECOMPILE_ATOMICS
+#define IF_TMB_PRECOMPILE_ATOMICS(...) __VA_ARGS__
 #else
-#define IF_TMB_PRECOMPILE(...)
+#define IF_TMB_PRECOMPILE_ATOMICS(...)
+#endif
+#ifdef HAVE_PRECOMPILED_ATOMICS
+#define CSKIP_ATOMIC(...) ;
+#else
+#define CSKIP_ATOMIC(...) __VA_ARGS__
 #endif
 
 /* Must come before Rinternals.h */
@@ -80,6 +85,10 @@ if (!(x)) {                                                             \
 #endif
 #include "EigenWarnings/DisableStupidWarnings"
 #endif
+/* We cannot use Eigen's parallel matrix multiply for AD types (GH390). */
+#ifndef EIGEN_DONT_PARALLELIZE
+#define EIGEN_DONT_PARALLELIZE
+#endif
 #include <Eigen/Dense>
 
 // Default: Include Eigen/Sparse normally
@@ -106,11 +115,16 @@ using SparseMatrix = SparseMatrix_rename<T, Flags, StorageIndex>;
 #endif
 #endif
 
+// TMBad config variables
+/** \brief Use deterministic hash codes for tape optimizer ? */
+TMB_EXTERN bool tmbad_deterministic_hash;
+
 /* Include the CppAD library. (Always turn off debug for cppad) */
 #undef NDEBUG
 #define NDEBUG 1
 #include "cppad/cppad.hpp"
 #ifdef TMBAD_FRAMEWORK
+#define TMBAD_DETERMINISTIC_HASH tmbad_deterministic_hash
 #include "TMBad/TMBad.hpp"
 #include "TMBad/tmbad_allow_comparison.hpp"
 #include "TMBad/eigen_numtraits.hpp"
@@ -154,6 +168,11 @@ namespace TMBad {
   using std::isnan;
   bool isnan(const TMBad::ad_aug &x)CSKIP({ return isnan(x.Value()); })
 }
+// Add missing numeric_limits specialization
+namespace std {
+template<>
+struct numeric_limits<TMBad::ad_aug> : numeric_limits<TMBad::Scalar> { };
+}
 #endif
 
 /* Include the R library _after_ Eigen and CppAD. Otherwise, the R
@@ -199,7 +218,9 @@ namespace CppAD{
 #include "lgamma.hpp"  // harmless
 #include "start_parallel.hpp"
 #include "tmbutils/newton.hpp" // Newton solver + Laplace used by TransformADFunObject
+#ifndef TMB_SKINNY
 #include "tmb_core.hpp"
+#endif
 #include "distributions_R.hpp"
 #include "convenience.hpp"    // Requires besselK
 #include "tmbutils/tmbutils_extra.hpp"
@@ -216,3 +237,5 @@ using Eigen::Array;
 // Nothing more to precompile
 #undef CSKIP
 #define CSKIP(...) __VA_ARGS__
+#undef CSKIP_ATOMIC
+#define CSKIP_ATOMIC(...) __VA_ARGS__
